package nl.amis.timelinerss;

import java.io.IOException;
import java.io.InputStream;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.HashMap;
import java.util.Map;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Characters;
import javax.xml.stream.events.XMLEvent;


public class RSSFeedParser {
    static final String TITLE = "title";
    static final String DESCRIPTION = "description";
    static final String CHANNEL = "channel";
    static final String LANGUAGE = "language";
    static final String COPYRIGHT = "copyright";
    static final String LINK = "link";
    static final String AUTHOR = "author";
    static final String ENTRY_CREATOR = "creator";
    static final String ITEM = "item";
    static final String PUB_DATE = "pubDate";
    static final String GUID = "guid";

    final URL url;

    public RSSFeedParser(String feedUrl) {
        try {
            this.url = new URL(feedUrl);
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }
    }

    public Feed readFeed() throws XMLStreamException {

        Map<String, String> values = new HashMap<String, String>();
        Feed feed = null;
        try {
            boolean isFeedHeader = true;
            // Set header values intial to the empty string

            // First create a new XMLInputFactory
            XMLInputFactory inputFactory = XMLInputFactory.newInstance();
            // Setup a new eventReader
            InputStream in = read();
            XMLEventReader eventReader = inputFactory.createXMLEventReader(in);
            // Read the XML document
            while (eventReader.hasNext()) {
                XMLEvent event = eventReader.nextEvent();
                if (event.isStartElement()) {
                    String localPart =
                        event.asStartElement().getName().getLocalPart();
                    if (ITEM.equals(localPart)) {
                        if (isFeedHeader) {
                            isFeedHeader = false;
                            feed =  new Feed(values.get("title"), values.get("link"), values.get("description"),
                                             values.get("language"), values.get("copyright"),
                                             values.get("pubDate")
                                             );
                            // remove values read for previous message to prepare for next one
                        }
                        event = eventReader.nextEvent();
                        values.clear();
                    }
                    if (DESCRIPTION.equals(localPart) ||
                        TITLE.equals(localPart) || LINK.equals(localPart) ||
                        GUID.equals(localPart) || LANGUAGE.equals(localPart) ||
                        AUTHOR.equals(localPart) ||
                        ENTRY_CREATOR.equals(localPart) ||
                        PUB_DATE.equals(localPart) ||
                        COPYRIGHT.equals(localPart)) {
                        values.put(localPart,
                                   getCharacterData(event, eventReader));
                    }
                } else if (event.isEndElement()) {
                    String localPart =
                        event.asEndElement().getName().getLocalPart();
                    if (ITEM.equals(localPart)) {
                        FeedMessage message = new FeedMessage();
                        message.setAuthor(values.get("creator"));
                        message.setDescription(values.get("description"));
                        message.setLink(values.get("link"));
                        message.setTitle(values.get("title"));
                        message.setPubDate(values.get("pubDate"));
                        feed.getMessages().add(message);
                        event = eventReader.nextEvent();
                        // remove values read for previous message to prepare for next one
                        values.clear();
                        continue;
                    }
                }
            }
        } catch (XMLStreamException e) {
            throw new RuntimeException(e);
        }
        return feed;
    }

    private String getCharacterData(XMLEvent event,
                                    XMLEventReader eventReader) throws XMLStreamException {
        String result = "";
        event = eventReader.nextEvent();
        if (event instanceof Characters) {
            result = event.asCharacters().getData();
        }
        return result;
    }

    private InputStream read() {
        try {
            return url.openStream();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
